﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

namespace M3dNamespace
{
    /// Author: Dennis Lang - 2009
    /// http://home.comcast.net/~lang.dennis/
    /// 
    /// <summary>
    /// Manage D3d render state parameters.
    /// </summary>
    public class D3dRenderState
    {
        public bool AlphaBlendEnable              = false;
        public BlendOperation AlphaBlendOperation = BlendOperation.Add;
        public Blend AlphaDestination             = Blend.SourceAlpha;
        public Compare AlphaFunction              = Compare.Equal;
        public Blend AlphaSource                  = Blend.SourceAlpha;
        public bool AlphaTestEnable               = false;
        public Color BlendFactor                  = Color.Black;
        public Blend DestinationBlend             = Blend.SourceAlpha;
        public BlendOperation BlendOperation      = BlendOperation.Add;
        public Blend SourceBlend                  = Blend.SourceAlpha;

        public Color Ambient                      = Color.White;
        public bool AntiAliasedLineEnable         = false;
        public bool Clipping                      = false;
        public bool ColorVertex                   = false;
        public Cull CullMode                      = Cull.None;
        public bool DitherEnable                  = false;
        public FillMode FillMode                  = FillMode.Solid;
        public bool LocalViewer                   = false;
        public bool NormalizeNormals              = false;
        public float PointSize                    = 2.0f;
        public ShadeMode ShadeMode                = ShadeMode.Flat;

        public Color FogColor                     = Color.Red;
        public bool FogEnable                     = false;
        public float FogEnd                       = 0.0f;
        public float FogStart                     = 0.0f;
        public bool RangeFogEnable                = false;

        public bool SpecularEnable                = false;
        public bool StencilEnable                 = false;

        public bool ZBufferEnable                 = false;
        public bool ZBufferWriteEnable            = false;
        public Compare ZBufferFunction            = Compare.Greater;

        public D3dRenderState()
        {
        }

        public void GetRenderState(Device d3dDevice)
        {
            this.AlphaBlendEnable    = d3dDevice.RenderState.AlphaBlendEnable;
            this.AlphaBlendOperation = d3dDevice.RenderState.AlphaBlendOperation;
            this.AlphaDestination    = d3dDevice.RenderState.AlphaDestinationBlend;
            this.AlphaFunction       = d3dDevice.RenderState.AlphaFunction;
            this.AlphaSource         = d3dDevice.RenderState.AlphaSourceBlend;
            this.AlphaTestEnable     = d3dDevice.RenderState.AlphaTestEnable;
            this.BlendFactor         = d3dDevice.RenderState.BlendFactor;
            this.DestinationBlend    = d3dDevice.RenderState.DestinationBlend;
            this.BlendOperation      = d3dDevice.RenderState.BlendOperation;
            this.SourceBlend         = d3dDevice.RenderState.SourceBlend;

            this.Ambient               = d3dDevice.RenderState.Ambient;
            this.AntiAliasedLineEnable = d3dDevice.RenderState.AntiAliasedLineEnable;
            this.Clipping              = d3dDevice.RenderState.Clipping;
            this.ColorVertex           = d3dDevice.RenderState.ColorVertex;
            this.CullMode              = d3dDevice.RenderState.CullMode;
            this.DitherEnable          = d3dDevice.RenderState.DitherEnable;
            this.FillMode              = d3dDevice.RenderState.FillMode;
            this.LocalViewer           = d3dDevice.RenderState.LocalViewer;
            this.NormalizeNormals      = d3dDevice.RenderState.NormalizeNormals;
            this.PointSize             = d3dDevice.RenderState.PointSize;
            this.ShadeMode             = d3dDevice.RenderState.ShadeMode;

            this.FogColor              = d3dDevice.RenderState.FogColor;
            this.FogEnable             = d3dDevice.RenderState.FogEnable;
            this.FogEnd                = d3dDevice.RenderState.FogEnd;
            this.FogStart              = d3dDevice.RenderState.FogStart;
            this.RangeFogEnable        = d3dDevice.RenderState.RangeFogEnable;

            this.SpecularEnable        = d3dDevice.RenderState.SpecularEnable;
            this.StencilEnable         = d3dDevice.RenderState.StencilEnable;

            this.ZBufferEnable         = d3dDevice.RenderState.ZBufferEnable;
            this.ZBufferWriteEnable    = d3dDevice.RenderState.ZBufferWriteEnable;
            this.ZBufferFunction       = d3dDevice.RenderState.ZBufferFunction;
        }

        public void SetRenderState(Device d3dDevice)
        {
            d3dDevice.RenderState.AlphaBlendEnable      = this.AlphaBlendEnable;
            d3dDevice.RenderState.AlphaBlendOperation   = this.AlphaBlendOperation;
            d3dDevice.RenderState.AlphaDestinationBlend = this.AlphaDestination;
            d3dDevice.RenderState.AlphaFunction         = this.AlphaFunction;
            d3dDevice.RenderState.AlphaSourceBlend      = this.AlphaSource;
            d3dDevice.RenderState.AlphaTestEnable       = this.AlphaTestEnable;
            d3dDevice.RenderState.BlendFactor           = this.BlendFactor;
            d3dDevice.RenderState.DestinationBlend      = this.DestinationBlend;
            d3dDevice.RenderState.BlendOperation        = this.BlendOperation;
            d3dDevice.RenderState.SourceBlend           = this.SourceBlend;

            d3dDevice.RenderState.Ambient               = this.Ambient;
            d3dDevice.RenderState.AntiAliasedLineEnable = this.AntiAliasedLineEnable;
            d3dDevice.RenderState.Clipping              = this.Clipping;
            d3dDevice.RenderState.ColorVertex           = this.ColorVertex;
            d3dDevice.RenderState.CullMode              = this.CullMode;
            d3dDevice.RenderState.DitherEnable          = this.DitherEnable;
            d3dDevice.RenderState.FillMode              = this.FillMode;
            d3dDevice.RenderState.LocalViewer           = this.LocalViewer;
            d3dDevice.RenderState.NormalizeNormals      = this.NormalizeNormals;
            d3dDevice.RenderState.PointSize             = this.PointSize;
            d3dDevice.RenderState.ShadeMode             = this.ShadeMode;

            d3dDevice.RenderState.FogColor              = this.FogColor;
            d3dDevice.RenderState.FogEnable             = this.FogEnable;
            d3dDevice.RenderState.FogEnd                = this.FogEnd;
            d3dDevice.RenderState.FogStart              = this.FogStart;
            d3dDevice.RenderState.RangeFogEnable        = this.RangeFogEnable;

            d3dDevice.RenderState.SpecularEnable     = this.SpecularEnable;
            d3dDevice.RenderState.StencilEnable      = this.StencilEnable;

            d3dDevice.RenderState.ZBufferEnable      = this.ZBufferEnable;
            d3dDevice.RenderState.ZBufferWriteEnable = this.ZBufferWriteEnable;
            d3dDevice.RenderState.ZBufferFunction    = this.ZBufferFunction;
        }


        private string boolStr = "True,False";
        private string shadeModeStr = "Flat,Gouraud,Phong";
        private string blendOpStr = "Add,Subtract,RevSubtract,Min,Max";
        private string blendStr = "Zero,One,SourceColor,SourceColor,InvSourceColor,SourceAlpha,InvSourceAlpha,DestinationAlpha,InvDestinationAlpha,DestinationColor,InvDestinationColor,SourceAlphaSat,BothSourceAlpha,BothInvSourceAlpha,BlendFactor,InvBlendFactor";
        private string colorStr = "Red,Green,Blue,Grey,Black,White,Yellow,Orange";
        private string fillModeStr = "Point,WireFrame,Solid";
        private string cullStr = "None,Clockwise,CounterClockwise";
        private string compareStr = "Never,Less,Equal,LessEqual,Greater,NotEqual,GreaterEqual,Always";

        /// <summary>
        /// Populuate Listview with RenderState values
        /// </summary>
        /// <param name="d3dView">ListView to be populated</param>
        public void DisplayRenderState(ref ListView d3dView)
        {
            d3dView.Items.Clear();

            d3dView.Items.Add("AlphaBlendEnable").SubItems.Add(this.AlphaBlendEnable.ToString()).Tag = boolStr;
            d3dView.Items.Add("AlphaBlendOperation").SubItems.Add(this.AlphaBlendOperation.ToString()).Tag = blendOpStr;
            d3dView.Items.Add("AlphaDestination").SubItems.Add(this.AlphaDestination.ToString()).Tag = blendStr;
            d3dView.Items.Add("AlphaFunction").SubItems.Add(this.AlphaFunction.ToString());
            d3dView.Items.Add("AlphaSource").SubItems.Add(this.AlphaSource.ToString()).Tag = blendStr;
            d3dView.Items.Add("AlphaTestEnable").SubItems.Add(this.AlphaTestEnable.ToString()).Tag = boolStr;
            d3dView.Items.Add("BlendFactor").SubItems.Add(this.BlendFactor.ToString());
            d3dView.Items.Add("DestinationBlend").SubItems.Add(this.DestinationBlend.ToString()).Tag = blendStr;
            d3dView.Items.Add("BlendOperation").SubItems.Add(this.BlendOperation.ToString()).Tag = blendOpStr;
            d3dView.Items.Add("SourceBlend").SubItems.Add(this.SourceBlend.ToString()).Tag = blendStr;
            d3dView.Items.Add("---");

            d3dView.Items.Add("Ambient").SubItems.Add(this.Ambient.ToString()).Tag = colorStr;
            d3dView.Items.Add("AntiAliasedLineEnable").SubItems.Add(this.AntiAliasedLineEnable.ToString()).Tag = boolStr;
            d3dView.Items.Add("Clipping").SubItems.Add(this.Clipping.ToString()).Tag = boolStr;
            d3dView.Items.Add("ColorVertex").SubItems.Add(this.ColorVertex.ToString());
            d3dView.Items.Add("CullMode").SubItems.Add(this.CullMode.ToString()).Tag = cullStr;
            d3dView.Items.Add("DitherEnable").SubItems.Add(this.DitherEnable.ToString()).Tag = boolStr;
            d3dView.Items.Add("FillMode").SubItems.Add(this.FillMode.ToString()).Tag = fillModeStr;
            d3dView.Items.Add("LocalViewer").SubItems.Add(this.LocalViewer.ToString()).Tag = boolStr;
            d3dView.Items.Add("NormalizeNormals").SubItems.Add(this.NormalizeNormals.ToString()).Tag = boolStr;
            d3dView.Items.Add("PointSize").SubItems.Add(this.PointSize.ToString());
            d3dView.Items.Add("ShadeMode").SubItems.Add(this.ShadeMode.ToString()).Tag = shadeModeStr;
            d3dView.Items.Add("---");

            d3dView.Items.Add("FogColor").SubItems.Add(this.FogColor.ToString()).Tag = colorStr;
            d3dView.Items.Add("FogEnable").SubItems.Add(this.FogEnable.ToString()).Tag = boolStr;
            d3dView.Items.Add("FogEnd").SubItems.Add(this.FogEnd.ToString());
            d3dView.Items.Add("FogStart").SubItems.Add(this.FogStart.ToString());
            d3dView.Items.Add("RangeFogEnable").SubItems.Add(this.RangeFogEnable.ToString()).Tag = boolStr;
            d3dView.Items.Add("---");

            d3dView.Items.Add("SpecularEnable").SubItems.Add(this.SpecularEnable.ToString()).Tag = boolStr;
            d3dView.Items.Add("StencilEnable").SubItems.Add(this.StencilEnable.ToString()).Tag = boolStr;
            d3dView.Items.Add("---");

            d3dView.Items.Add("ZBufferEnable").SubItems.Add(this.ZBufferEnable.ToString()).Tag = boolStr;
            d3dView.Items.Add("ZBufferWriteEnable").SubItems.Add(this.ZBufferWriteEnable.ToString()).Tag = boolStr;
            d3dView.Items.Add("ZBufferFunction").SubItems.Add(this.ZBufferFunction.ToString()).Tag = compareStr;
        }

        /// <summary>
        /// Extract values from ListView and set internal state
        /// </summary>
        /// <param name="d3dView">ListView previous populated via call to DisplayRenderState(listView)</param>
        public D3dRenderState UpdateRenderState(ListView d3dView)
        {
            // parse d3dView and populate rstate
            foreach (ListViewItem item in d3dView.Items)
            {
                string p = item.SubItems[0].Text;

                try
                {
                    if (item.SubItems.Count > 1)
                    {
                        string v = item.SubItems[1].Text;

                        switch (p)
                        {
                            case "AlphaBlendEnable":
                                this.AlphaBlendEnable = bool.Parse(v);
                                break;

                            case "AlphaBlendOperation":
                                this.AlphaBlendOperation = (BlendOperation)Enum.Parse(typeof(BlendOperation), v);
                                break;

                            case "AlphaDestination":
                                this.AlphaDestination = (Blend)Enum.Parse(typeof(Blend), v);
                                break;

                            case "AlphaFunction":
                                // this.AlphaFunction = Compare.Parse(v);
                                break;

                            case "AlphaSource":
                                this.AlphaSource = (Blend)Enum.Parse(typeof(Blend), v);
                                break;

                            case "AlphaTestEnable":
                                this.AlphaTestEnable = bool.Parse(v);
                                break;

                            case "BlendFactor":
                                this.BlendFactor = Color.FromName(v);
                                break;

                            case "DestinationBlend":
                                this.DestinationBlend = (Blend)Enum.Parse(typeof(Blend), v);
                                break;

                            case "BlendOperation":
                                this.BlendOperation = (BlendOperation)Enum.Parse(typeof(BlendOperation), v);
                                break;

                            case "SourceBlend":
                                this.SourceBlend = (Blend)Enum.Parse(typeof(Blend), v);
                                break;

                            case "Ambient":
                                this.Ambient = Color.FromName(v);
                                break;

                            case "AntiAliasedLineEnable":
                                this.AntiAliasedLineEnable = bool.Parse(v);
                                break;

                            case "Clipping":
                                this.Clipping = bool.Parse(v);
                                break;

                            case "ColorVertex":
                                this.ColorVertex = bool.Parse(v);
                                break;

                            case "CullMode":
                                this.CullMode = (Cull)Enum.Parse(typeof(Cull), v);
                                break;

                            case "DitherEnable":
                                this.DitherEnable = bool.Parse(v);
                                break;

                            case "FillMode":
                                this.FillMode = (FillMode)Enum.Parse(typeof(FillMode), v);
                                break;

                            case "LocalViewer":
                                this.LocalViewer = bool.Parse(v);
                                break;

                            case "NormalizeNormals":
                                this.NormalizeNormals = bool.Parse(v);
                                break;

                            case "PointSize":
                                this.PointSize = float.Parse(v);
                                break;

                            case "ShadeMode":
                                this.ShadeMode = (ShadeMode)Enum.Parse(typeof(ShadeMode), v);
                                break;

                            case "FogColor":
                                this.FogColor = Color.FromName(v);
                                break;

                            case "FogEnable":
                                this.FogEnable = bool.Parse(v);
                                break;

                            case "FogEnd":
                                this.FogEnd = float.Parse(v);
                                break;

                            case "FogStart":
                                this.FogStart = float.Parse(v);
                                break;

                            case "RangeFogEnable":
                                this.RangeFogEnable = bool.Parse(v);
                                break;

                            case "SpecularEnable":
                                this.SpecularEnable = bool.Parse(v);
                                break;

                            case "StencilEnable":
                                this.StencilEnable = bool.Parse(v);
                                break;

                            case "ZBufferEnable":
                                this.ZBufferEnable = bool.Parse(v);
                                break;

                            case "ZBufferWriteEnable":
                                this.ZBufferWriteEnable = bool.Parse(v);
                                break;

                            case "ZBufferFunction":
                                this.ZBufferFunction = (Compare)Enum.Parse(typeof(Compare), v);
                                break;
                        }
                    }
                }
                catch
                {
                    System.Diagnostics.Debug.WriteLine("Failed to decode " + p);
                }
            }

            return this;
        }


        /// <summary>
        /// Try and handle key event on ListView UI.
        /// </summary>
        /// <param name="d3dView"></param>
        /// <param name="e"></param>
        /// <returns>True if item changed</returns>
        public  bool KeyUp(ListView d3dView, KeyEventArgs e)
        {
            Point mousePoint = new Point(Control.MousePosition.X, Control.MousePosition.Y);
            Point d3dPoint = d3dView.PointToClient(mousePoint);

            ListViewItem item = null;

            if (d3dView.SelectedItems.Count > 0)
                item = d3dView.SelectedItems[0];
            if (item == null)
                item = d3dView.GetItemAt(d3dPoint.X, d3dPoint.Y);

            if (item != null && item.SubItems.Count > 1)
            {
                string choices = item.SubItems[1].Tag as string;
                string v = item.SubItems[1].Text;
                // this.Status.Text = item.SubItems[0].Text + " = " + v;

                if (choices == boolStr)
                {
                    switch (e.KeyCode)
                    {
                        case Keys.T:
                            item.SubItems[1].Text = choices.Split(',')[0];
                            break;
                        case Keys.F:
                            item.SubItems[1].Text = choices.Split(',')[1];
                            break;
                    }

                    // UpdateRenderState();
                    return true;
                }
            }

            return false;
        }
    }
}
