﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Windows;
using System.Windows.Forms;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using System.Runtime.InteropServices;

namespace M3dNamespace
{
    /// Author: Dennis Lang - 2009
    /// http://home.comcast.net/~lang.dennis/
    /// 
    /// <summary>
    /// Create d3d device and support on/off screen rendering via delegates
    /// </summary>
    public class D3dRenderTexture
	{
        private D3d d3d;
        private D3d.DrawDelegate drawDelegate;
        private Size size;

        private Texture renderTex;
        private Surface renderSur;
        private RenderToSurface renderToSur;

        public D3dRenderTexture()
        {
        }

        /// <summary>
        /// Setup D3d device and delegate draw method.
        /// Specify view rotation and whether lights are on.
        /// </summary>
        public void SetRender(D3d inD3d, D3d.DrawDelegate inDrawDelegate, Size inSize)
        {
            this.d3d = inD3d;
            this.drawDelegate = inDrawDelegate;
            this.size = inSize;
        }

        /// <summary>
        /// Clear various textures and surfaces
        /// </summary>
        public void Clear()
        {
            if (this.renderTex != null)
            {
                this.renderSur.Dispose();
                this.renderToSur.Dispose();
                this.renderTex.Dispose();
                this.renderTex = null;
            }
        }

        /// <summary>
        /// Draw d3d objects, assumes z-buffer is enabled and draws delegates first
        /// with background grid drawn last.
        /// </summary>
        /// <param name="d3d"></param>
        private void DrawObjects(D3d d3d)
        {
            this.DrawToTexture(d3d);
        }

        public Texture GetTexture()
        {
            return this.renderTex;
        }

        public Texture DrawToTexture(D3d d3d)
        {
            if (renderTex == null)
            {
                int numLevels = 1;
                renderTex = new Texture(d3d.d3dDevice, this.size.Width, this.size.Height, numLevels,
                    Usage.RenderTarget, Format.A8R8G8B8, Pool.Default);

                renderSur = renderTex.GetSurfaceLevel(0);
                SurfaceDescription surDesc = renderSur.Description;

                // d3dDevice.CreateDepthStencilSurface(size.Width, size.Height, DepthFormat.D16, 
                //     MultiSampleType.None, 0, true);

                renderToSur = new RenderToSurface(d3d.d3dDevice,
                        surDesc.Width, surDesc.Height,
                        surDesc.Format, true, DepthFormat.D16);
            }

            renderToSur.BeginScene(renderSur);
            {
                if (this.drawDelegate != null)
                    this.drawDelegate(d3d);
            }

            renderToSur.EndScene(Filter.Linear);
            return renderTex;
        }
    }
}
