﻿
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Drawing;
using System.Windows;
using System.Windows.Forms;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using System.Runtime.InteropServices;

namespace M3dNamespace
{
    /// Author: Dennis Lang - 2009
    /// http://home.comcast.net/~lang.dennis/
    /// 
    /// Draw some basic objects.
    /// 
    public class D3dShape
    {
        public Color color = Color.Red;

        // -----------------------------------------------------------------

        private Mesh sphere;
       
        public void BuildSphere(D3d d3d, float width)
        {
            int stacks = 16;
            int slicesAround = 16;
            sphere = Mesh.Sphere(d3d.d3dDevice, width/2.0f, slicesAround, stacks);
            sphere.ComputeNormals();
        }

        public void ClearSphere()
        {
            this.sphere = null;
        }

        public void DrawShere(D3d d3d, Vector3 translate)
        {
            if (this.sphere != null)
            {
                d3d.d3dDevice.SetTexture(0, null);      // clear texture else its color is used.
                Material[] m = new Material[2];
                m[0] = d3d.d3dDevice.Material;          // save material
                m[1].Ambient = Color.Black;
                m[1].Diffuse = this.color;
                d3d.d3dDevice.Material = m[1];

                // Save color, set new color, draw, restore original color
                Color c = d3d.d3dDevice.RenderState.Ambient;
                Matrix w = d3d.d3dDevice.Transform.World;
                {
                    d3d.d3dDevice.RenderState.Ambient = Color.White;

                    Matrix translationMatrix = Matrix.Identity;
                    translationMatrix.Translate(translate);
                    d3d.d3dDevice.Transform.World = translationMatrix * w;

                    this.sphere.DrawSubset(0);
                }
                d3d.d3dDevice.RenderState.Ambient = c;  // restore color
                d3d.d3dDevice.Material = m[0];          // restore material
                d3d.d3dDevice.Transform.World = w;      // restore transform
            }
        }

        // -----------------------------------------------------------------

        // public Mesh plane;
        private CustomVertex.PositionNormalTextured[] planeList = null;

        private Texture planeTexture;

        public void BuildPlane(D3d d3d, Size bgSize)
        {
            // plane = Mesh.Box(d3d.d3dDevice, bgSize.Width, bgSize.Height, 1.0f);

            this.planeTexture = null;
           
            planeList = new CustomVertex.PositionNormalTextured[4];

            //  -1, 1 +-----+ 1, 1
            //         \ -
            //             \
            //  -1,-1 +-----+ 1, -1
            //

            float x = 0.5f;
            float y = 0.5f;
            float z = -0.5f;
            planeList[0].Position =  new Vector3(-x, -y, 0.0f);
            planeList[0].Normal = new Vector3(-x, -y, z);
            planeList[0].Tu       = 0;
            planeList[0].Tv       = 0;

            planeList[1].Position = new Vector3(x, -y, 0.0f);
            planeList[1].Normal = new Vector3(x, -y, z);
            planeList[1].Tu       = 0;
            planeList[1].Tv       = 1;

            planeList[2].Position = new Vector3(-x, y, 0.0f);
            planeList[2].Normal = new Vector3(-x, y, z);
            planeList[2].Tu       = 1;
            planeList[2].Tv       = 0;

            planeList[3].Position = new Vector3(x, y, 0.0f);
            planeList[3].Normal = new Vector3(x, y, z);
            planeList[3].Tu       = 1;
            planeList[3].Tv       = 1;
        }

        public void ClearPlane()
        {
            this.planeList = null;
            this.planeTexture = null;
        }

        public void SetPlaneTexture(D3d d3d, string texFilename)
        {
            d3d.d3dDevice.SetTexture(0, null);
            if (texFilename != null && texFilename.Length > 0)
            {
                this.planeTexture = TextureLoader.FromFile(d3d.d3dDevice, texFilename);
                System.Diagnostics.Debug.WriteLine(this.planeTexture.ToString());
                this.color = Color.White;
            }
        }
        public void SetPlaneTexture(D3d d3d, Bitmap texImage)
        {
            d3d.d3dDevice.SetTexture(0, null);
            if (texImage != null && texImage.Width > 0)
            {
                this.planeTexture = Texture.FromBitmap(d3d.d3dDevice, texImage, Usage.Dynamic, Pool.Default);
                System.Diagnostics.Debug.WriteLine(this.planeTexture.ToString());
                this.color = Color.White;
            }
        }

        public void DrawPlane(D3d d3d)
        {
            if (this.planeList != null)
            {
                Material[] m = new Material[2];
                m[0] = d3d.d3dDevice.Material;          // save material
                m[1].Ambient = Color.Black;
                m[1].Diffuse = this.color;
                d3d.d3dDevice.Material = m[1];

                d3d.d3dDevice.SetTexture(0, planeTexture);      

                // Save color, set new color, draw, restore original color
                Color c = d3d.d3dDevice.RenderState.Ambient;
                Matrix w = d3d.d3dDevice.Transform.World;
                {
                    d3d.d3dDevice.RenderState.Ambient = Color.White;

                    Matrix translationMatrix = Matrix.Identity;
                    d3d.d3dDevice.Transform.World = translationMatrix; //  *w;

                    // this.plane.DrawSubset(0);
                    d3d.d3dDevice.VertexFormat = CustomVertex.PositionNormalTextured.Format;
                    d3d.d3dDevice.DrawUserPrimitives(PrimitiveType.TriangleStrip, planeList.Length -2, planeList);
                }
                d3d.d3dDevice.RenderState.Ambient = c;  // restore color
                d3d.d3dDevice.Material = m[0];          // restore material
                d3d.d3dDevice.Transform.World = w;      // restore transform
            }
        }
    }
}
